package kernel.peripherals;

import util.Signal;
import cc.Term.TerminalSize;
import lib.TermWriteable;
import util.Vec.Vec2;
import util.Color;

class Screen implements TermWriteable{
	private final nativ:cc.periphs.Monitor.Monitor;
	private final addr:String;


	@:allow(kernel.peripherals)
	public function new(nativePeripherals: cc.periphs.Monitor.Monitor,addr: String) {
		this.nativ = nativePeripherals;
		this.addr = addr;

		KernelEvents.instance.on("monitor_resize",params -> {
			if (params[1] == this.addr){
				_onResize.emit(null);
			}
		});

		setTextScale(0.5);
	}

	public function getAddr(): String {
		return this.addr;
	}

	public var onResize(get,null):SignalReadonly<Vec2<Int>>;
	private final _onResize:Signal<Vec2<Int>> = new Signal();

	function get_onResize():SignalReadonly<Vec2<Int>> {
		return _onResize;
	}

	public function getTextScale(): Float {
		return nativ.getTextScale();
	}

	public function setTextScale(scale: Float) {
		nativ.setTextScale(scale);
	}

	public function write(text:String) {
        nativ.write(text);
    }

	public function scroll(y:Int) {
        nativ.scroll(y);
    }

	public function getCursorPos():Vec2<Int> {
		var rtn = nativ.getCursorPos();
        return {
            x: rtn.x - 1,
            y: rtn.y - 1
        }
	}

	public function setCursorPos(x:Int, y:Int) {
        nativ.setCursorPos(x + 1,y + 1);
    }

	public function getCursorBlink():Bool {
        return nativ.getCursorBlink();
	}

	public function setCursorBlink(blink:Bool) {
        nativ.setCursorBlink(blink);
    }

	public function getSize():Vec2<Int> {
		// FIXME: this will not compile. Has to be changes upstream
		var size:TerminalSize = nativ.getSize();
		return {
			x: size.width,
			y: size.height,
		}
	}

	public function clear() {
        nativ.clear();
    }

	public function clearLine() {
        nativ.clearLine();
    }

	public function getTextColor():Color {
		return ColorConvert.ccToColor(nativ.getTextColor());
	}

	public function setTextColor(colour:Color) {
        nativ.setTextColor(ColorConvert.colorToCC(colour));
    }

	public function getBackgroundColor():Color {
		return ColorConvert.ccToColor(nativ.getBackgroundColor());
	}

	public function setBackgroundColor(color:Color) {
        nativ.setBackgroundColor(ColorConvert.colorToCC(color));
    }

	public function isColor():Bool {
		return nativ.isColor();
	}
}

