package util;

interface SignalReadonly<T> {
    public function on(cb: T->Void):SignalListner<T>;
    public function once(cb: T->Void):SignalListner<T>;
    public function remove(id:SignalListner<T>):Void;
}

class SignalListner<T> {
    private final callback:T->Void;

    @:allow(util.Signal)
    private final once:Bool;

    @:allow(util.Signal)
    private function new(callback: T->Void,?once: Bool = false) {
        this.callback = callback;
        this.once = once;
    }

    @:allow(util.Signal)
    private function invoke(params: T) {
        if (callback != null){
            callback(params);
        }
    }
}

/**
    Simple event system for one event type other than EventBus which has multiple events.
**/
class Signal<T> implements SignalReadonly<T>{
    public final listner:Array<SignalListner<T>> = new Array();

    public function new() {}

    public function on(cb: T->Void):SignalListner<T> {
        var l = new SignalListner<T>(cb,false);
        listner.push(l);
        return l;
    }

    public function once(cb: T->Void):SignalListner<T>  {
        var l = new SignalListner<T>(cb,true);
        listner.push(l);
        return l;
    }

    public function emit(data: T) {
        for (cb in listner){
            cb.invoke(data);
            if (cb.once){
                listner.remove(cb);
            }
        }
    }

    public function remove(id:SignalListner<T> ) {
        listner.remove(id);
    }
}
